/*
 * C
 *
 * Copyright 2021 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */

// -----------------------------------------------------------------------------
// Includes
// -----------------------------------------------------------------------------

#include <stdio.h>
#include <string.h>

#include "LLFEATURE_INPUTSTREAM.h"
#include "bsp_util.h"
#include "sni.h"
#include "fsl_debug_console.h"
#include "usart_feature_loader.h"
#include "power_manager.h"

// -----------------------------------------------------------------------------
// Macros and defines
// -----------------------------------------------------------------------------

/*
 * @brief Declare the buffer where read the feature (.fo) file.
 */
BSP_DECLARE_BUFFER(FEATURE_LOADER_RAM_BUFFER)

// -----------------------------------------------------------------------------
// Globals
// -----------------------------------------------------------------------------

static volatile uint8_t* ptr;

// -----------------------------------------------------------------------------
// FeatureInputStream API
// -----------------------------------------------------------------------------

void Java_com_microej_kernel_FeatureInputStream_closeFeature(void)
{
	// erase buffer memory: ready to be used to install a new feature
	// (erase only the ELF signature: the 4 first bytes)
	memset((void*)FEATURE_LOADER_RAM_BUFFER_START, 0, FEATURE_SIGNATURE_SIZE);
}

void Java_com_microej_kernel_FeatureInputStream_init(void)
{
	PRINTF("Feature loader buffer address: 0x%x\n", FEATURE_LOADER_RAM_BUFFER_START);

	/* Start usart feature loader */
	USART_Feature_Loader_Init();

	// force to invalidate RAM
	Java_com_microej_kernel_FeatureInputStream_closeFeature();
}

jint Java_com_microej_kernel_FeatureInputStream_readIntoArray(jbyte* array, jint offset, jint l)
{
	memcpy((void*)(array + offset), (const void*)ptr, l);
	ptr += l;
	return l;
}

jint Java_com_microej_kernel_FeatureInputStream_readByte(void)
{
	return (jint)(*ptr++);
}

jboolean Java_com_microej_kernel_FeatureInputStream_isFeatureAvailable (void)
{
	if (FEATURE_SIGNATURE_VALUE == *((uint32_t *)FEATURE_LOADER_RAM_BUFFER_START))
	{
		// "ELF" header is available: feature is ready to be installed

		// reset feature pointer
		ptr = (uint8_t*)FEATURE_LOADER_RAM_BUFFER_START;

		// Power Up KF ram
		power_manager_kfbuffer_power_on();

		return JTRUE;
	}

	return JFALSE;
}

// -----------------------------------------------------------------------------
// EOF
// -----------------------------------------------------------------------------
