/*
 * C
 *
 * Copyright 2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
#if !defined DISPLAY_DMA_H
# define DISPLAY_DMA_H

// -----------------------------------------------------------------------------
// Includes
// -----------------------------------------------------------------------------

#include "mej_debug.h"

#include "display_framebuffer.h"

// -----------------------------------------------------------------------------
// Macros and Defines
// -----------------------------------------------------------------------------

#define DISPLAY_DMA_ENABLED       ( APP_BUFFER_COUNT > 1 )

// -----------------------------------------------------------------------------
// Public functions
// -----------------------------------------------------------------------------

/*
 * @brief: Initialize the DMA synchronization between the framebuffers
 *
 * @param[in] framebuffers: display framebuffers
 */
void display_dma_initialize(const framebuffer_t * framebuffers[]);

/*
 * @brief: Starts a DMA transfert
 *
 * @param[in] src: source framebuffer
 * @param[in] dst: destination framebuffer
 * @param[in] ymin: first line to transfert
 * @param[in] ymax: last line to transfert + 1
 */
void display_dma_start(framebuffer_t *src, framebuffer_t *dst, int ymin, int ymax);

#endif // !defined DISPLAY_DMA_H

// -----------------------------------------------------------------------------
// EOF
// -----------------------------------------------------------------------------

