/*
 * Java
 *
 * Copyright 2020 MicroEJ Corp. All rights reserved.
 * Use of this source code is governed by a BSD-style license that can be found with this software.
 */
package com.microej.demo.watch.util.style.text;

import java.io.IOException;

import ej.microui.display.GraphicsContext;
import ej.mwt.util.Alignment;
import ej.vector.font.VectorFont;
import ej.vector.font.VectorFontPainter;

/**
 * Draws a TrueType text using the {@link GraphicsContext}'s color.
 */
public class PlainVectorTextStyle implements VectorTextStyle {

	/**
	 * Plain vector text style singleton to avoid creating several ones.
	 */
	public static final VectorTextStyle PLAIN_VECTOR_TEXT_STYLE = new PlainVectorTextStyle();

	@Override
	public void drawText(GraphicsContext g, String text, VectorFont font, int fontHeight, int contentWidth,
			int contentHeight, int horizontalAlignment, int verticalAlignment) {
		int stringWidth = font.stringWidth(fontHeight, text);
		int x = Alignment.computeLeftX(stringWidth, 0, contentWidth, horizontalAlignment);
		int y = Alignment.computeTopY(fontHeight, 0, contentHeight, verticalAlignment);
		VectorFontPainter.drawString(g, font, fontHeight, text, x, y);
	}

	@Override
	public void close() throws IOException {
		// Nothing to do.
	}

}
