/*
 * Java
 *
 * Copyright 2019-2020 MicroEJ Corp. All rights reserved.
 * Use of this source code is governed by a BSD-style license that can be found with this software.
 */
package com.microej.path.mascot;

import com.microej.demo.watch.util.path.VectorGraphics;
import com.microej.vivante.utils.SvgImage;
import com.verisilicon.vivante.vg_lite_matrix;

/**
 * A vector graphics that represents the MicroEJ mascot.
 */
public class Mascot extends VectorGraphics {

	private static final int ANGLE_STEP = 5;

	private static final int ZOOM_STEPS = 25;

	private static final float DEFAULT_SCALE = 0.5f;

	private boolean zoomOut;

	private float scale;

	private int scaleCount;

	private int angle;

	private int rotationCenterX;

	private int rotationCenterY;

	/**
	 * Creates the MicroEJ mascot by describing its path.
	 */
	public Mascot() {
		super(SvgImage.getSvgImage("mascot.svg")); //$NON-NLS-1$
		this.scale = DEFAULT_SCALE;
	}

	/**
	 * Moves to the next frame of the mascot animation.
	 */
	public void nextFrame() {
		this.angle += ANGLE_STEP;

		vg_lite_matrix matrix = getMatrix();
		vg_lite_matrix.vg_lite_identity(matrix);

		vg_lite_matrix.vg_lite_translate(this.rotationCenterX, this.rotationCenterY, matrix);

		if (this.zoomOut) {
			this.scale *= 1.05f;
			vg_lite_matrix.vg_lite_scale(this.scale, this.scale, matrix);
			if (-ZOOM_STEPS == --this.scaleCount) {
				this.zoomOut = false;
			}
		} else {
			this.scale *= 0.9523809f;
			vg_lite_matrix.vg_lite_scale(this.scale, this.scale, matrix);
			if (ZOOM_STEPS == ++this.scaleCount) {
				this.zoomOut = true;
			}
		}

		vg_lite_matrix.vg_lite_rotate(this.angle, matrix);
		vg_lite_matrix.vg_lite_translate(-this.rotationCenterX, -this.rotationCenterY, matrix);
	}

	/**
	 * Sets the rotation center point coordinates.
	 *
	 * @param x
	 *            the rotation center x-coordinate to set.
	 * @param y
	 *            the rotation center y-coordinate to set.
	 *
	 */
	public void setRotationCenter(int x, int y) {
		this.rotationCenterX = x;
		this.rotationCenterY = y;
		nextFrame();
	}
}
