/*
 * Java
 *
 * Copyright 2019-2021 MicroEJ Corp. All rights reserved.
 * Use of this source code is governed by a BSD-style license that can be found with this software.
 */
package com.microej.widget;

import com.microej.path.mascot.Mascot;
import com.microej.path.mascot.Nxp;

import ej.microui.display.GraphicsContext;
import ej.mwt.Widget;
import ej.mwt.animation.Animation;
import ej.mwt.util.Size;

/**
 * A widget that animates the MicroEJ vector mascot.
 */
public class VectorMascot extends Widget {

	private static final float NXP_LOGO_SPEED_FACTOR = 3;

	private Mascot mascot;

	private Nxp nxp;

	private Animation animation;

	@Override
	protected void renderContent(GraphicsContext g, int contentWidth, int contentHeight) {
		if (this.nxp.isMovingOnFront()) {
			this.mascot.render(g);
			this.nxp.render(g);
		} else {
			this.nxp.render(g);
			this.mascot.render(g);
		}
	}

	@Override
	protected void computeContentOptimalSize(Size size) {
		// nothing to do: take available size
	}

	@Override
	protected void onShown() {
		super.onShown();
		Animation animation = new Animation() {

			@Override
			public boolean tick(long currentTimeMillis) {
				if (VectorMascot.this.mascot != null) {
					VectorMascot.this.mascot.nextFrame();
					VectorMascot.this.nxp.nextFrame();
				}
				requestRender();
				return true;
			}
		};
		this.animation = animation;
		getDesktop().getAnimator().startAnimation(animation);
	}

	@Override
	protected void onHidden() {
		getDesktop().getAnimator().stopAnimation(this.animation);
		super.onHidden();
	}

	@Override
	protected void onLaidOut() {
		super.onLaidOut();
		closeMascot();

		this.mascot = new Mascot();
		this.mascot.setRotationCenter(getAbsoluteX() + getWidth() / 2, getAbsoluteY() + getHeight() / 2);
		this.nxp = new Nxp(getWidth(), NXP_LOGO_SPEED_FACTOR);
	}

	@Override
	protected void onDetached() {
		super.onDetached();
		closeMascot();
	}

	private void closeMascot() {
		Mascot mascot = this.mascot;
		if (mascot != null) {
			mascot.close();
			this.mascot = null;
		}

		Nxp nxp = this.nxp;
		if (nxp != null) {
			nxp.close();
			this.nxp = null;
		}
	}
}
