/*
 * Java
 *
 * Copyright 2016 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.cssgenerator.selector;

import java.io.StringWriter;
import java.util.Hashtable;
import java.util.Map;

import org.w3c.css.sac.AttributeCondition;
import org.w3c.css.sac.Condition;

/**
 *
 */
public abstract class AbstractAttributeSelector implements Selector {

	private final Map<String, String> attributeSelectorVars = new Hashtable<String, String>();

	@Override
	public String generate(Condition c, StringWriter writer) throws IllegalArgumentException {
		AttributeCondition condition = this.get(c);
		String localName = condition.getLocalName();
		String value = condition.getValue();
		String varPrefix = getSelectorVariablePrefix();
		StringBuilder attributeSelector = new StringBuilder(varPrefix).append(localName).append(value);
		String attributeSelectorVar = this.attributeSelectorVars.get(attributeSelector.toString());
		if (attributeSelectorVar != null) {
			return attributeSelectorVar;
		}

		StringBuilder attributeSelectorVarSb = new StringBuilder(varPrefix).append("_").append(localName).append("_")
				.append(value);
		writer.append(this.getClassName()).append(" ").append(attributeSelectorVarSb);
		writer.append(" = new ");
		writer.append(this.getClassName()).append('(');
		writer.append("\"").append(localName).append("\", ");
		writer.append("\"").append(value).append("\"");
		writer.append(");\n");
		this.attributeSelectorVars.put(attributeSelector.toString(), attributeSelectorVarSb.toString());
		return attributeSelectorVarSb.toString();
	}

	private AttributeCondition get(Condition condition) throws IllegalArgumentException {
		if (!this.matches(condition)) {
			throw new IllegalArgumentException("Given condition is not an AttributeCondition");
		}
		return (AttributeCondition) condition;
	}

	@Override
	public String generate(org.w3c.css.sac.Selector s, StringWriter writer) throws IllegalArgumentException {
		throw new RuntimeException();
	}

	@Override
	public boolean matches(org.w3c.css.sac.Selector selector) {
		return false;
	}

	protected abstract String getSelectorVariablePrefix();

}
