/*
 * Java
 *
 * Copyright 2016 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.description;

import java.io.StringWriter;
import java.util.LinkedHashMap;

/**
 * The class {@code StyleDescription} represents a CSS style description, for example a description of the
 * <code>color</code> CSS declaration.
 */
public interface StyleDescription {

	/**
	 * Gets the class name that matches the represented CSS property.
	 *
	 * @return the fully qualified name of the class that handled this CSS property, or <code>null</code> if no class
	 *         matched the CSS property.
	 */
	String getClassName();

	/**
	 * Generates Java source code for the represented CSS declaration.
	 *
	 * @param variableName
	 *            the name of the Java style variable.
	 * @param rules
	 *            a map of all available CSS rules, useful to determine what content to generate.
	 * @param writer
	 *            the writer to append the Java source code to.
	 */
	void generateStyle(String variableName, LinkedHashMap<String, String> rules, StringWriter writer);

}
