/*
 * Java
 *
 * Copyright 2016-2019 MicroEJ Corp. All rights reserved.
 * MicroEJ Corp. PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.description;

public class Type {

	public enum Basetype {
		Boolean("boolean"), Byte("byte"), Short("short"), Integer("int"), Long("long"), Float("float"), Double(
				"double"), Object("java.lang.Object"), String("java.lang.String"), RefId("reference"), Code("int");

		private final String name;

		private Basetype(String name) {
			this.name = name;
		}

		/**
		 * Gets the name of the base type.
		 *
		 * @return the name.
		 */
		public String getName() {
			return this.name;
		}
	}

	public static final Type BOOLEAN = new Type(Basetype.Boolean);
	public static final Type BYTE = new Type(Basetype.Byte);
	public static final Type SHORT = new Type(Basetype.Short);
	public static final Type INTEGER = new Type(Basetype.Integer);
	public static final Type LONG = new Type(Basetype.Long);
	public static final Type FLOAT = new Type(Basetype.Float);
	public static final Type DOUBLE = new Type(Basetype.Double);
	public static final Type STRING = new Type(Basetype.String);
	public static final Type REFERENCE = new Type(Basetype.RefId, "java.lang.String");
	/**
	 * Inlined Code (Javascript)
	 */
	public static final Type CODE = new Type(Basetype.Code);

	private final Basetype basetype;
	private final String typeName;

	public Type(Basetype basetype) {
		this(basetype, basetype.getName());
	}

	public Type(Basetype basetype, String typeName) {
		this.basetype = basetype;
		this.typeName = typeName;
	}

	/**
	 * Gets the basetype.
	 *
	 * @return the basetype.
	 */
	public Basetype getBasetype() {
		return this.basetype;
	}

	/**
	 * Gets the typeName.
	 *
	 * @return the typeName.
	 */
	public String getTypeName() {
		return this.typeName;
	}

}
