/*
 * Java
 *
 * Copyright 2016 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.generator;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Comparator;
import java.util.List;
import java.util.Set;

import com.is2t.giml.xml.model.Attribute;
import com.is2t.giml.xml.model.Node;

/**
 * The GIML generator utility class
 */
public class GIMLGeneratorUtils {

	/**
	 * Sorts the attributes of the given node by name.
	 * 
	 * @param node
	 * 			the node whose attributes will be sorted.
	 * @return
	 * 			the array of the sorted attributes.
	 */
	public static Attribute[] sortNodeAttributesByName(Node node) {
		Set<Attribute> attributes = node.getAttributes();
		Attribute[] attributesArray = attributes.toArray(new Attribute[attributes.size()]);
		Arrays.sort(attributesArray, new Comparator<Attribute>() {
			@Override
			public int compare(Attribute o1, Attribute o2) {
				return o1.getName().toLowerCase().compareTo(o2.getName().toLowerCase());
			}
		});

		return attributesArray;
	}

	/**
	 * Sorts the attributes names of the given node.
	 * 
	 * @param node
	 * 			the node whose attributes will be sorted.
	 * @return
	 * 			the array of the sorted attributes names.
	 */
	public static String[] sortAttributesNamesByName(Node node) {
		Attribute[] attributes = sortNodeAttributesByName(node);
		String[] attributeNames = new String[attributes.length];
		int i = -1;
		for(Attribute attribute : attributes){
			attributeNames[++i] = attribute.getName();
		}
		return attributeNames;
	}

	/**
	 * Sorts the children of the given node by name.
	 * 
	 * @param node
	 * 			the node whose children will be sorted.
	 * @return
	 * 			the list of the sorted children.
	 */
	public static List<Node> sortNodeChildrenByName(Node node) {
		return sortNodesByName(node.getChildren());
	}

	/**
	 * Sorts the given nodes list by name.
	 * 
	 * @param nodes
	 * 			the nodes list to sort.
	 * @return
	 * 			the sorted nodes list.
	 */
	public static List<Node> sortNodesByName(List<Node> nodes) {
		List<Node> childrenList = new ArrayList<Node>();
		Node[] childrenArray = nodes.toArray(new Node[nodes.size()]);
		// sort attributes by name
		Arrays.sort(childrenArray, new Comparator<Node>() {
			@Override
			public int compare(Node o1, Node o2) {
				return o1.getName().toLowerCase().compareTo(o2.getName().toLowerCase());
			}
		});
		for (Node child : childrenArray) {
			childrenList.add(child);
		}
		return childrenList;
	}

	/**
	 * Sorts the given string collection.
	 * 
	 * @param strings
	 * 			the string collection to sort.
	 * @return
	 * 			the array of the sorted strings
	 */
	public static String[] sortIngnoreCase(Collection<String> strings) {
		String[] stringArray = strings.toArray(new String[strings.size()]);
		Arrays.sort(stringArray, new Comparator<String>() {
			@Override
			public int compare(String o1, String o2) {
				return o1.toLowerCase().compareTo(o2.toLowerCase());
			}
		});
		return stringArray;
	}

}
