/*
 * Java
 *
 * Copyright 2016-2017 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.generator.test;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import org.junit.Assert;
import org.junit.Test;

import com.is2t.giml.binary.GIMLWriter;
import com.is2t.giml.description.DescriptionFactory;
import com.is2t.giml.generator.GIMLGenerator;
import com.is2t.giml.generator.GIMLGeneratorException;
import com.is2t.giml.xml.model.Attribute;
import com.is2t.giml.xml.model.Node;
import com.is2t.giml.xml.parser.exception.XMLParserException;

public class TestGeneratorWrittenDataSequence {

	private static final String ID_ATTRIBUTE_VALUE1 = "myFirstID";
	private static final String ID_ATTRIBUTE_VALUE2 = "mysecondid";
	private static final String ROOT_ID = "rootId";
	private static final String GENERATED_ROOT_IDENTIFIER = "0";
	private static final String STRING_1_VALUE = "_(Bla)";
	private static final String STRING_2_VALUE = "Hello";
	private static final String STRING_3_VALUE = "gettext(Hello world)";
	private static final String STRING_4_VALUE = "foo";
	private static final String BOOLEAN_VALUE = "false";


	public TestGeneratorWrittenDataSequence() {
		DescriptionFactory.addDescription(RootNodeDescription.TAG, new RootNodeDescription());
		DescriptionFactory.addDescription(SimpleNodeDescription.TAG, new SimpleNodeDescription());
		DescriptionFactory.addDescription(NodeWithNotOrderedElementsDescription.TAG,
				new NodeWithNotOrderedElementsDescription());
		DescriptionFactory.addDescription(NodeWithOrderedArrayElementsDescription.TAG,
				new NodeWithOrderedArrayElementsDescription());
	}

	@Test
	public void test01() throws NullPointerException, XMLParserException, IOException, GIMLGeneratorException {

		ByteArrayOutputStream arrayOutputStream = new ByteArrayOutputStream();
		DataOutputStream out = new DataOutputStream(arrayOutputStream);
		GIMLWriter writer = new MyGIMLWriter(out);

		HashSet<Attribute> simpleNodeAttributes = new HashSet<>();
		simpleNodeAttributes.add(new Attribute("text1", STRING_1_VALUE));
		simpleNodeAttributes.add(new Attribute("text2", STRING_2_VALUE));
		simpleNodeAttributes.add(new Attribute("text3", STRING_3_VALUE));
		simpleNodeAttributes.add(new Attribute("text4", STRING_4_VALUE));
		Node simpleNode = new Node(SimpleNodeDescription.TAG, simpleNodeAttributes);

		HashSet<Attribute> dockNodeAttributes = new HashSet<>();
		dockNodeAttributes.add(new Attribute("horizontal", BOOLEAN_VALUE));
		dockNodeAttributes.add(new Attribute("id", ID_ATTRIBUTE_VALUE2));
		List<Node> firstNodeChild = new ArrayList<>();
		firstNodeChild.add(simpleNode);
		Node firstNode = new Node("first", new HashSet<Attribute>(), firstNodeChild);
		Node centerNode = new Node("center", new HashSet<Attribute>(), firstNodeChild);
		List<Node> dockNodeChildren = new ArrayList<>();
		dockNodeChildren.add(firstNode);
		dockNodeChildren.add(centerNode);
		Node dock = new Node("dock", dockNodeAttributes, dockNodeChildren);
		List<Node> element1Child = new ArrayList<>();
		element1Child.add(dock);
		Node element1 = new Node("element", new HashSet<Attribute>(), element1Child);

		List<Node> listElements = new ArrayList<>();
		listElements.add(element1);
		listElements.add(element1);
		HashSet<Attribute> listAttributes = new HashSet<>();
		listAttributes.add(new Attribute("id", ID_ATTRIBUTE_VALUE1));
		Node list = new Node("list", listAttributes, listElements);

		List<Node> elementNodeChild = new ArrayList<>();
		elementNodeChild.add(list);
		Node element = new Node(RootNodeDescription.ELEMENT, new HashSet<Attribute>(), elementNodeChild);

		HashSet<Attribute> rootNodeAttributes = new HashSet<>();
		rootNodeAttributes.add(new Attribute("id", ROOT_ID));
		List<Node> rootNodeChild = new ArrayList<>();
		rootNodeChild.add(element);
		Node root = new Node(RootNodeDescription.TAG, rootNodeAttributes, rootNodeChild);

		List<Node> gimlRootChildren = new ArrayList<>();
		gimlRootChildren.add(root);
		Node gimlRoot = new Node("root", new HashSet<Attribute>(), gimlRootChildren);

		File output = Helper.createTmpDir();
		GIMLGenerator gimlGenerator = new GIMLGenerator(gimlRoot, output);

		gimlGenerator.generate(writer);

		//check the written data sequence
		ByteArrayInputStream arrayInputStream = new ByteArrayInputStream(arrayOutputStream.toByteArray());
		DataInputStream in = new DataInputStream(arrayInputStream);

		// mark position
		Assert.assertTrue(in.readInt() == 0); //identifier 0
		// factory id
		Assert.assertTrue(in.readInt() == 0);
		// root id attribute for description init code generation
		Assert.assertTrue(in.readUTF().equals(GENERATED_ROOT_IDENTIFIER));
		// root id attribute value for element registration
		Assert.assertTrue(in.readUTF().equals(GENERATED_ROOT_IDENTIFIER));
		// list factory id
		Assert.assertTrue(in.readInt() == 1);
		// list id attribute
		Assert.assertTrue(in.readUTF().equals(ID_ATTRIBUTE_VALUE1));
		// list element count
		Assert.assertTrue(in.readInt() == 2);
		for (int i = -1; ++i < 2;) {
			// dock factory id
			Assert.assertTrue(in.readInt() == 2);
			// dock horizontal attribute
			Assert.assertTrue(in.readBoolean() == false);
			// dock id attribute for element registration
			Assert.assertTrue(in.readUTF().equals(ID_ATTRIBUTE_VALUE2));
			// simple node factory id
			Assert.assertTrue(in.readInt() == 3);
			// simple node text1 attribute (nls id)
			Assert.assertTrue(in.readInt() == (1));
			// simple node text2 attribute (non nls string)
			Assert.assertTrue(in.readUTF().equals(STRING_2_VALUE));
			// simple node text3 attribute (nls id)
			Assert.assertTrue(in.readInt() == (2));
			// simple node text4 attribute (non nls string)
			Assert.assertTrue(in.readUTF().equals(STRING_4_VALUE));
			// simple node factory id
			Assert.assertTrue(in.readInt() == 3);
			// simple node text1 attribute (nls id)
			Assert.assertTrue(in.readInt() == (1));
			// simple node text2 attribute (non nls string)
			Assert.assertTrue(in.readUTF().equals(STRING_2_VALUE));
			// // simple node text3 attribute (nls id)
			Assert.assertTrue(in.readInt() == (2));
			// simple node text4 attribute (non nls string)
			Assert.assertTrue(in.readUTF().equals(STRING_4_VALUE));
		}

	}



}
