/*
 * Java
 *
 * Copyright 2019 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.jsgenerator;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import jdk.nashorn.internal.ir.FunctionNode;
import jdk.nashorn.internal.ir.LexicalContext;
import jdk.nashorn.internal.ir.Statement;
import jdk.nashorn.internal.ir.VarNode;
import jdk.nashorn.internal.ir.visitor.NodeVisitor;

/**
 * Visitor applied before Java code generation to accumulate data on javascript
 * functions.
 * <li>handle locals of a function</li>
 */
public class JavaScriptSourceFunctionAnalyzer extends NodeVisitor<LexicalContext> {

	private final ArrayList<String> locals;

	public JavaScriptSourceFunctionAnalyzer() {
		super(new LexicalContext());
		locals = new ArrayList<>();
	}

	/**
	 * API Analyze a JavaScript function.
	 */
	public AnalyzedFunction generate(FunctionNode node) {
		List<Statement> statements = node.getBody().getStatements();
		List<com.is2t.java.nodes.Statement> generatedStatementsVect = new ArrayList<>();
		for (Statement statement : statements) {
			statement.accept(this);
		}
		AnalyzedFunction af = new AnalyzedFunction(locals.toArray(new String[locals.size()]));
		return af;
	}

	@Override
	public boolean enterVarNode(VarNode varNode) {
		String localName = varNode.getName().getName();
		if (!locals.contains(localName)) {
			locals.add(localName);
		}

		return true;
	}

	public static class AnalyzedFunction {

		private final String[] locals;

		public AnalyzedFunction(String[] locals) {
			this.locals = locals;
		}

		/**
		 * Gets the locals.
		 *
		 * @return the locals.
		 */
		public String[] getLocals() {
			return locals;
		}

		public boolean isLocal(String varName) {
			return Arrays.asList(locals).contains(varName);
		}
	}
}
