/*
 * Java
 *
 * Copyright 2016-2017 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.runtime.reader.test;

import java.io.IOException;

import com.is2t.giml.runtime.reader.GIMLReaderFactory;
import com.is2t.testsuite.support.CheckHelper;

import ej.components.dependencyinjection.ServiceLoaderFactory;
import ej.giml.GIMLReader;
import ej.nls.NLS;

/**
 *
 */
public class GIMLReaderCompactedTest {

	public static void main(String[] args) throws IOException {
		GIMLReaderFactory readerFactory = ServiceLoaderFactory.getServiceLoader().getService(GIMLReaderFactory.class);
		GIMLReader reader = readerFactory.newGIMLReader("/com/is2t/giml/runtime/reader/test/testfile.giml");

		testReader(GIMLReaderCompactedTest.class, reader);

		CheckHelper.endCheck(GIMLReaderCompactedTest.class);
	}

	public static void testReader(Class clazz, GIMLReader reader) {
		reader.initialize(new SimpleNLS());

		checkVarUInt(clazz, reader);
		checkVarSInt(clazz, reader);
		checkVarLong(clazz, reader);
		checkNLSString(clazz, reader);
		checkNonNLSString(clazz, reader);
		checkNonNLSUTF8String(clazz, reader);
		checkFloat(clazz, reader);
		checkDouble(clazz, reader);
		checkBoolean(clazz, reader);
		checkSeekPosition(clazz, reader);

	}

	public static void checkVarUInt(Class clazz, GIMLReader reader) {
		CheckHelper.check(clazz, "err0", reader.readVarUInt(), 0);
		CheckHelper.check(clazz, "err1", reader.readVarUInt(), 27);

		int value = 0b1111111;
		CheckHelper.check(clazz, "err1_" + value, reader.readVarUInt(), value);
		CheckHelper.check(clazz, "err2_" + value, reader.readVarUInt(), value + 1);
		CheckHelper.check(clazz, "err3_" + value, reader.readVarUInt(), value + 27);

		value = 0b111111_11111111;
		CheckHelper.check(clazz, "err1_" + value, reader.readVarUInt(), value);
		CheckHelper.check(clazz, "err2_" + value, reader.readVarUInt(), value + 1);
		CheckHelper.check(clazz, "err3_" + value, reader.readVarUInt(), value + 27);

		value = 0b11111_11111111_11111111;
		CheckHelper.check(clazz, "err1_" + value, reader.readVarUInt(), value);
		CheckHelper.check(clazz, "err2_" + value, reader.readVarUInt(), value + 1);
		CheckHelper.check(clazz, "err3_" + value, reader.readVarUInt(), value + 27);

		value = 0b1111_11111111_11111111_11111111;
		CheckHelper.check(clazz, "err1_" + value, reader.readVarUInt(), value);
		CheckHelper.check(clazz, "err2_" + value, reader.readVarUInt(), value + 1);
		CheckHelper.check(clazz, "err3_" + value, reader.readVarUInt(), value + 27);

		value = 0b11111111_11111111_11111111_11111111;
		CheckHelper.check(clazz, "err1_" + value, reader.readVarUInt(), value);

		CheckHelper.check(clazz, "err4", reader.readVarUInt(), 0b0101010);
		CheckHelper.check(clazz, "err5", reader.readVarUInt(), 0b10010010110110);
		CheckHelper.check(clazz, "err6", reader.readVarUInt(), 0b101010100110011100110);
		CheckHelper.check(clazz, "err7", reader.readVarUInt(), 0b11100010101010110001111011010010);
		CheckHelper.check(clazz, "err8", reader.readVarUInt(), 0b10100010101010110001111011010010);
		CheckHelper.check(clazz, "err9", reader.readVarUInt(), 0b01100010101010110001111011010010);
	}

	public static void checkVarSInt(Class clazz, GIMLReader reader) {
		int value = 0;
		int negValue;

		CheckHelper.check(clazz, "err1_" + value, reader.readVarSInt(), value);
		CheckHelper.check(clazz, "err2_" + value, reader.readVarSInt(), value + 27);

		value = 0b111111;
		negValue = -(value + 1);
		CheckHelper.check(clazz, "err1_" + value, reader.readVarSInt(), value);
		CheckHelper.check(clazz, "err1_" + negValue, reader.readVarSInt(), negValue);
		CheckHelper.check(clazz, "err1_" + value + 1, reader.readVarSInt(), value + 1);
		CheckHelper.check(clazz, "err1_" + (negValue - 1), reader.readVarSInt(), negValue - 1);

		// Test 3
		value = 0b11111_11111111;
		negValue = -(value + 1);
		CheckHelper.check(clazz, "err1_" + value, reader.readVarSInt(), value);
		CheckHelper.check(clazz, "err1_" + negValue, reader.readVarSInt(), negValue);
		CheckHelper.check(clazz, "err1_" + value + 1, reader.readVarSInt(), value + 1);
		CheckHelper.check(clazz, "err1_" + (negValue - 1), reader.readVarSInt(), negValue - 1);

		// Test 4
		value = 0b1111_11111111_11111111;
		negValue = -(value + 1);
		CheckHelper.check(clazz, "err1_" + value, reader.readVarSInt(), value);
		CheckHelper.check(clazz, "err1_" + negValue, reader.readVarSInt(), negValue);
		CheckHelper.check(clazz, "err1_" + value + 1, reader.readVarSInt(), value + 1);
		CheckHelper.check(clazz, "err1_" + (negValue - 1), reader.readVarSInt(), negValue - 1);

		// Test 5
		value = 0b111_11111111_11111111_11111111;
		negValue = -(value + 1);
		CheckHelper.check(clazz, "err1_" + value, reader.readVarSInt(), value);
		CheckHelper.check(clazz, "err1_" + negValue, reader.readVarSInt(), negValue);
		CheckHelper.check(clazz, "err1_" + value + 1, reader.readVarSInt(), value + 1);
		CheckHelper.check(clazz, "err1_" + (negValue - 1), reader.readVarSInt(), negValue - 1);

		// Test 6
		CheckHelper.check(clazz, "err1_" + Integer.MIN_VALUE, reader.readVarSInt(), Integer.MIN_VALUE);
		CheckHelper.check(clazz, "err1_" + Integer.MAX_VALUE, reader.readVarSInt(), Integer.MAX_VALUE);
		CheckHelper.check(clazz, "err1_" + -1, reader.readVarSInt(), -1);

		CheckHelper.check(clazz, "err1_" + 0b1111_1111111_1111111_1111111_1111110, reader.readVarSInt(),
				0b1111_1111111_1111111_1111111_1111110);
		CheckHelper.check(clazz, "err1_" + 0b1111_1111111_1111111_1111110_1111111, reader.readVarSInt(),
				0b1111_1111111_1111111_1111110_1111111);
		CheckHelper.check(clazz, "err1_" + 0b1111_1111111_1111110_1111111_1111111, reader.readVarSInt(),
				0b1111_1111111_1111110_1111111_1111111);
		CheckHelper.check(clazz, "err1_" + 0b1111_1111110_1111111_1111111_1111111, reader.readVarSInt(),
				0b1111_1111110_1111111_1111111_1111111);
		CheckHelper.check(clazz, "err1_" + 0b1110_1111111_1111111_1111111_1111111, reader.readVarSInt(),
				0b1110_1111111_1111111_1111111_1111111);
		CheckHelper.check(clazz, "err1_" + 0b1010_1111111_1111111_1111111_1111111, reader.readVarSInt(),
				0b1010_1111111_1111111_1111111_1111111);
		CheckHelper.check(clazz, "err1_" + 0b0110_1111111_1111111_1111111_1111111, reader.readVarSInt(),
				0b0110_1111111_1111111_1111111_1111111);
	}

	public static void checkVarLong(Class clazz, GIMLReader reader) {
		long value;

		value = 0;
		CheckHelper.check(clazz, "errlong_" + value, reader.readVarLong(), value);
		value = 1;
		CheckHelper.check(clazz, "errlong_" + value, reader.readVarLong(), value);
		value = -1;
		CheckHelper.check(clazz, "errlong_" + value, reader.readVarLong(), value);
		value = Long.MAX_VALUE;
		CheckHelper.check(clazz, "errlong_" + value, reader.readVarLong(), value);
		value = Long.MAX_VALUE - 1;
		CheckHelper.check(clazz, "errlong_" + value, reader.readVarLong(), value);
		value = Long.MIN_VALUE;
		CheckHelper.check(clazz, "errlong_" + value, reader.readVarLong(), value);
		value = Long.MIN_VALUE + 1;
		CheckHelper.check(clazz, "errlong_" + value, reader.readVarLong(), value);
	}

	public static void checkNLSString(Class clazz, GIMLReader reader) {
		String str;
		str = reader.readString();
		CheckHelper.check(clazz, "errNLSString1", str, "0");
		str = reader.readString();
		CheckHelper.check(clazz, "errNLSString2", str, "54");
		str = reader.readString();
		CheckHelper.check(clazz, "errNLSString3", str, "63");
		str = reader.readString();
		CheckHelper.check(clazz, "errNLSString4", str, "64");
	}

	public static void checkNonNLSString(Class clazz, GIMLReader reader) {
		String str;
		str = reader.readString();
		CheckHelper.check(clazz, "errString1", str, "yahoo!");
		str = reader.readString();
		CheckHelper.check(clazz, "errString2", str, "yop");
		str = reader.readString();
		CheckHelper.check(clazz, "errString3", str, "yahoo!");
		str = reader.readString();
		CheckHelper.check(clazz, "errString4", str,
				"This is a very long string. This is a very long string. This is a very long string. This is a very long string. This is a very long string. ");
	}

	public static void checkNonNLSUTF8String(Class clazz, GIMLReader reader) {
		String str;
		str = reader.readString();
		CheckHelper.check(clazz, "errStringUTF8", str, "abcdefg\u0178hijkl");
	}

	private static void checkFloat(Class clazz, GIMLReader reader) {
		float value = reader.readFloat();
		CheckHelper.check(clazz, "checkFloat1", value, 1.03245f);
	}

	private static void checkDouble(Class clazz, GIMLReader reader) {
		double value = reader.readDouble();
		CheckHelper.check(clazz, "checkDouble1", value, 1.03245);
	}

	private static void checkBoolean(Class clazz, GIMLReader reader) {
		boolean value = reader.readBoolean();
		CheckHelper.check(clazz, "checkBoolean1", value, true);
		value = reader.readBoolean();
		CheckHelper.check(clazz, "checkBoolean2", value, false);
	}

	private static void checkSeekPosition(Class clazz, GIMLReader reader) {
		reader.seek(RootIds.id1);
		int value = reader.readVarUInt();
		CheckHelper.check(clazz, "checkSeekPosition1", value, 91);

		reader.seek(RootIds.id3);
		value = reader.readVarUInt();
		CheckHelper.check(clazz, "checkSeekPosition3", value, 93);

		reader.seek(RootIds.id9);
		value = reader.readVarUInt();
		CheckHelper.check(clazz, "checkSeekPosition3", value, 99);

		reader.seek(RootIds.id2);
		value = reader.readVarUInt();
		CheckHelper.check(clazz, "checkSeekPosition3", value, 92);

		reader.seek(RootIds.id5);
		value = reader.readVarUInt();
		CheckHelper.check(clazz, "checkSeekPosition3", value, 95);

		reader.seek(RootIds.id4);
		value = reader.readVarUInt();
		CheckHelper.check(clazz, "checkSeekPosition3", value, 94);

		reader.seek(RootIds.id8);
		value = reader.readVarUInt();
		CheckHelper.check(clazz, "checkSeekPosition3", value, 98);

		reader.seek(RootIds.id6);
		value = reader.readVarUInt();
		CheckHelper.check(clazz, "checkSeekPosition3", value, 96);

		reader.seek(RootIds.id7);
		value = reader.readVarUInt();
		CheckHelper.check(clazz, "checkSeekPosition3", value, 97);

		reader.seek(RootIds.id5);
		value = reader.readVarUInt();
		CheckHelper.check(clazz, "checkSeekPosition3", value, 95);

		boolean ok = false;
		try {
			reader.seek(11);
			CheckHelper.check(clazz, "checkSeekPosition2", false);
		} catch (IllegalArgumentException e) {
			ok = true;
		}

		CheckHelper.check(clazz, "checkSeekPosition IllegalArgument", ok);
	}

	/**
	 * A simple NLS that returns a string that contains the messageId.
	 */
	static class SimpleNLS extends NLS {

		@Override
		public void setCurrentLocale(String locale) {
		}

		@Override
		public String getMessage(int messageID, String locale) {
			return getMessage(messageID);
		}

		@Override
		public String getMessage(int messageID) {
			return Integer.toString(messageID);
		}

		@Override
		public String getDisplayName(String locale) {
			return "";
		}

		@Override
		public String getCurrentLocale() {
			return "";
		}

		@Override
		public String[] getAvailableLocales() {
			return new String[0];
		}
	}

}
