/*
 * Java
 *
 * Copyright 2016 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.xml.generator.test.xmlgenerator;

import java.io.IOException;

import org.hamcrest.CoreMatchers;
import org.junit.Assert;
import org.junit.Test;

import com.is2t.giml.xml.generator.XMLGenerator;
import com.is2t.giml.xml.generator.XMLGeneratorOptions;
import com.is2t.giml.xml.generator.test.AbstractTest;
import com.is2t.giml.xml.model.Document;

/**
 * Unit tests for {@link XMLGenerator} class.
 */
public class XMLGeneratorTest extends AbstractTest {

	private static final String OTHER_EXTENSION = "other";

	@Test
	public void testDefaultXMLGeneratorCreation() throws IOException {
		Document document = new Document("doc", "doc");
		XMLGenerator xmlGenerator = new XMLGenerator(document);

		Assert.assertEquals(xmlGenerator.getOptions(), new XMLGeneratorOptions());
	}

	@Test
	public void testChangeOptions() throws IOException {
		Document document = new Document("doc", "doc");
		XMLGenerator xmlGenerator = new XMLGenerator(document);

		// Directly change an option
		XMLGeneratorOptions originalOptions = xmlGenerator.getOptions();
		originalOptions.setXMLFileExtension(OTHER_EXTENSION);
		Assert.assertThat(originalOptions, CoreMatchers.not(CoreMatchers.equalTo(new XMLGeneratorOptions())));

		// Set a new option object, equals options but not same instance
		XMLGeneratorOptions otherOptions = new XMLGeneratorOptions();
		otherOptions.setXMLFileExtension(OTHER_EXTENSION);
		xmlGenerator.setOptions(otherOptions);
		Assert.assertEquals(xmlGenerator.getOptions(), otherOptions);
		Assert.assertEquals(xmlGenerator.getOptions(), originalOptions);
		Assert.assertThat(xmlGenerator.getOptions(), CoreMatchers.not(CoreMatchers.sameInstance(originalOptions)));
	}

	@Test(expected = NullPointerException.class)
	public void testNullRootFile() {
		new XMLGenerator(null);
	}

}
