/*
 * Java
 *
 * Copyright 2016 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.xml.parser.exception;

import java.util.Iterator;
import java.util.Stack;

import com.is2t.giml.xml.model.Document;

/**
 * An instance of {@code XMLParserCircularException} is thrown to indicate that there exists a circular dependency
 * between documents. This circular dependency is caused by circular imports in documents.
 */
public class XMLParserCircularImportException extends XMLParserException {

	private static final long serialVersionUID = 2985265549365247169L;

	private static final String CIRCULAR_IMPORT_SEPARATOR = " > ";

	/**
	 * Constructs a new circular dependency exception with the specified detail message.
	 *
	 * @param message
	 *            the detail message.
	 */
	public XMLParserCircularImportException(String message) {
		super(message);
	}

	/**
	 * Constructs a new circular dependency exception with the specified details about circular imports in documents.
	 *
	 * @param circularDependencyDetails
	 *            a stack of document where there is a circular dependency in imports.
	 *
	 * @return a new circular dependency exception.
	 */
	public static XMLParserCircularImportException fromCircularDependencyDetails(
			Stack<Document> circularDependencyDetails) {
		StringBuilder builder = new StringBuilder();
		Stack<Document> copyStack = new Stack<Document>();
		copyStack.addAll(circularDependencyDetails);

		builder.append("Circular import detected: ");

		Document root = copyStack.pop();
		builder.append(root.getId());

		Iterator<Document> iterator = copyStack.iterator();
		Document next = iterator.hasNext() ? iterator.next() : null;
		while (next != null && !next.equals(root)) {
			builder.append(CIRCULAR_IMPORT_SEPARATOR);
			builder.append(next.getId());

			if (iterator.hasNext()) {
				next = iterator.next();
			} else {
				next = null;
			}
		}

		builder.append(CIRCULAR_IMPORT_SEPARATOR);
		builder.append(root.getId());

		return new XMLParserCircularImportException(builder.toString());
	}

}
