/*
 * Java
 *
 * Copyright 2009-2019 MicroEJ Corp. All rights reserved.
 * MicroEJ Corp. PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
 package com.is2t.java.nodes;

import com.is2t.java.JavaSourceGenerator;
import com.is2t.java.nodes.expressions.Operators;


public abstract class Expression extends Statement implements Operators {

	@Override
	public final void generateUsing(JavaSourceGenerator gen){
		gen.generateExpression(this);
	}

	public abstract void generateExpressionUsing(JavaSourceGenerator gen);


	public static void operatorToString(int id,StringBuffer stream){
		//convert the operator id to a string representation

		switch (id&0xFF){
		case Not : stream.append("!");break;
		case Plus : stream.append("+");break;
		case Minus : stream.append("-");break;
		case PreIncr:
		case PostIncr:
			stream.append("++");
			break;
		case PreDecr:
		case PostDecr:
			stream.append("--");
			break;
		case Multiply : stream.append("*");break;
		case Divide : stream.append("/");break;
		case Remainder : stream.append("%");break;
		case LeftShift : stream.append("<<");break;
		case RightShift : stream.append(">>");break;
		case UnsignedRightShift : stream.append(">>>");break;
		case LowerThan : stream.append("<");break;
		case GreaterThan : stream.append(">");break;
		case LowerThanOrEqual : stream.append("<=");break;
		case GreaterThanOrEqual : stream.append(">=");break;
		case Instanceof :  stream.append("instanceof");break;
		case Equal : stream.append("==");break;
		case NotEqual : stream.append("!=");break;
		case BitwiseAnd : stream.append("&");break;
		case BitwiseXor :stream.append("^");break;
		case BitwiseOr :stream.append("|");break;
		case BitwiseComplement: stream.append("~");break;
		case And : stream.append("&&");break;
		case Or : stream.append("||");break;
		case Assignment : stream.append("=");break;
		case AssignmentMultiply : stream.append("*=");break;
		case AssignmentDivide : stream.append("/=");break;
		case AssignmentRemainder : stream.append("%=");break;
		case AssignmentStringAdd :
		case AssignmentPlus : stream.append("+=");break;
		case AssignmentMinus : stream.append("-=");break;
		case AssignmentLeftShift : stream.append("<<=");break;
		case AssignmentRightShift :  stream.append(">>=");break;
		case AssignmentUnsignedRightShift :stream.append(">>>=");break;
		case AssignmentBitwiseAnd : stream.append("&=");break;
		case AssignmentBitwiseXor :  stream.append("^=");break;
		case AssignmentBitwiseOr :stream.append("|=");break;
		case QuestionColon : stream.append("?");break; //a fake value for consistency
		case StringAdd : stream.append("+");break;
		default : stream.append(">>??<<");
		}
	}
}
