package com.is2t.java.nodes;

import com.is2t.java.JavaSourceGenerator;

public class TypeReference extends JavaSourceNode{

	public static final int T_BOOLEAN 	= 1;
	public static final int T_BYTE	 	= 2;
	public static final int T_CHAR 		= 3;
	public static final int T_SHORT 	= 4;
	public static final int T_INT 		= 5;
	public static final int T_LONG 		= 6;
	public static final int T_FLOAT		= 7;
	public static final int T_DOUBLE	= 8;
	public static final int T_VOID 		= 9;
	public static final int T_REF 		= 10;



	public int dimensions;		// array dimension or 0 for baseType or Type
	public int id;

	public static int DimNoQualifier 	= 0;
	public static int DimQualifierRo 	= 1;
	public static int DimQualifierUpper 	= 2;
	/**
	 * IceTea dimensions qualifiers (optional)
	 * dimsQualifiers may be null when no dimensions is qualified, otherwise its length == dimension
	 */
	public int[] dimsQualifiers;


	// these following 2 fields are set only for Type or array of Type
	public char[][] fullyQualifiedName;		// declaration fullyQualifiedName
	public TypeDeclaration declaration;		// only if the type must be generated. Null for pre-existing type : java/lang/Object for example...

	public TypeReference(int basetypeId){
		this.id = basetypeId;
	}

	public TypeReference(TypeDeclaration declaration){
		this( declaration.fullyQualifiedName, 0);
		this.declaration = declaration;
	}

	public TypeReference(int basetypeId, int dimensions){
		this.id = basetypeId;
		this.dimensions = dimensions;
	}

	public TypeReference(TypeDeclaration declaration, int dimensions){
		this( declaration.fullyQualifiedName, dimensions);
		this.declaration = declaration;
	}


	// for Type or Type[] not to generate
	public TypeReference(char[][] fullyQualifiedName, int dimension){
		this.id = T_REF;
		// this.declaration = null; vm done
		this.dimensions = dimension;
		this.fullyQualifiedName = fullyQualifiedName;
	}

	public static TypeReference getIntReference() {
		return new TypeReference( T_INT );
	}

	public static TypeReference getBooleanReference() {
		return new TypeReference( T_BOOLEAN );
	}

	public static TypeReference getIntReference(int dims) {
		TypeReference type =  new TypeReference( T_INT );
		type.dimensions = dims;
		return type;
	}

	public static TypeReference getVoidReference() {
		return new TypeReference( T_VOID );
	}


	@Override
	public void generateUsing(JavaSourceGenerator gen) {
		gen.generateTypeReference(this);
	}

	public boolean isBasetypeOrArrayOfBasetype(){
		return id != T_REF;
	}

	public boolean isArray(){
		return dimensions > 0;
	}

	/*
	 * Use it only when nodes are totally initialized
	 */
	public boolean isPreExistingTypeOrArray(){
		return id == T_REF && declaration == null;
	}

	@Override
	public boolean equals( Object obj) {
		if (! (obj instanceof TypeReference)) {
			return false;
		}

		TypeReference type = (TypeReference) obj;
		if ( id != type.id  || dimensions != type.dimensions) {
			return false;
		}

		if ( id != T_REF ) {
			return true;
		}

		char[][] fullyName = fullyQualifiedName;
		char[][] objFullyName = type.fullyQualifiedName;
		if( fullyName.length != objFullyName.length) {
			return false;
		}

		for (int i = fullyName.length; --i >=0; ) {
			if (!new String(fullyName[i]).equals(new String(objFullyName[i]))) {
				return false;
			}
		}

		return true;
	}

	public void appendSuffixBaseType( StringBuilder sb){
		switch(id){
		case T_BYTE: 		sb.append("Byte"); break;
		case T_CHAR: 		sb.append( "Char"); break;
		case T_SHORT: 		sb.append( "Short"); break;
		case T_INT: 		sb.append( "Int"); break;
		case T_LONG: 		sb.append( "Long"); break;
		case T_BOOLEAN: 	sb.append( "Boolean"); break;
		case T_DOUBLE:		sb.append( "Double"); break;
		case T_FLOAT:		sb.append( "Float"); break;
		}
	}

	public boolean isAddressReference() {
		return false;
	}

	public void setDimQualifier(int dimensionIndex, int value){
		if(dimsQualifiers == null) {
			dimsQualifiers = new int[dimensions];
		}
		dimsQualifiers[dimensionIndex] = value;
	}

	public int getDimQualifier(int dimensionIndex){
		if(dimsQualifiers == null) {
			return DimNoQualifier;
		}
		return dimsQualifiers[dimensionIndex];
	}

	public boolean isFloat() {
		return id == T_FLOAT && dimensions == 0;
	}

	public boolean isDouble() {
		return id == T_DOUBLE && dimensions == 0;
	}
}
