/*
 * Java
 *
 * Copyright 2018-2019 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.support.security.keyfactory;

import java.security.KeyFactorySpi;
import java.security.PrivateKey;
import java.security.spec.EncodedKeySpec;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.KeySpec;

import com.is2t.support.security.NativePrivateKey;

import ej.sni.NativeException;
import ej.sni.SNI;

/**
 *
 */
public class NativeKeyFactorySpi extends KeyFactorySpi {
	private final String algorithm;
	private final int nativeAlgorithmId;

	/**
	 *
	 */
	public NativeKeyFactorySpi(int nativeAlgorithmId, String algorithm) {
		this.nativeAlgorithmId = nativeAlgorithmId;
		this.algorithm = algorithm;
	}

	@Override
	protected PrivateKey engineGeneratePrivate(KeySpec keySpec) throws InvalidKeySpecException {
		EncodedKeySpec encodedKeySpec = (EncodedKeySpec) keySpec;
		String format = encodedKeySpec.getFormat();
		byte[] encodedKey = encodedKeySpec.getEncoded();

		byte[] cFormat = new byte[format.length() + 1]; // add 1 for the '\0'
		SNI.toCString(format, cFormat);

		int nativeKeyLength = nativeGetPrivateKeyLength(nativeAlgorithmId, cFormat, encodedKey, encodedKey.length);
		byte[] nativeKeyData = new byte[nativeKeyLength];

		nativeGetPrivateKeyData(nativeAlgorithmId, cFormat, nativeKeyData, nativeKeyData.length, encodedKey,
				encodedKey.length);
		return new NativePrivateKey(nativeKeyData, algorithm);
	}

	/**
	 *
	 * @param algorithm
	 *            Null terminated string that describes the algorithm
	 * @return algorithm ID on success or -1 on error.
	 */
	native public static int nativeGetAlgorithmDescription(byte[] algorithm);

	/**
	 *
	 * @param nativeAlgorithmId
	 *            the algorithm ID
	 * @param format
	 *            the format of the key
	 * @param keyData
	 *            the resulting private key data
	 * @param keyDataLength
	 * @param encodedKey
	 *            the encoded private key
	 * @param encodedKeyLength
	 * @throws NativeException
	 */
	native private static void nativeGetPrivateKeyData(int nativeAlgorithmId, byte[] format, byte[] keyData,
			int keyDataLength, byte[] encodedKey, int encodedKeyLength)
					throws NativeException;

	/**
	 *
	 * @param nativeAlgorithmId
	 *            the algorithm ID
	 * @param encodedKeyLength
	 *            the length of the encoded Key
	 * @return the length of a native key corresponding to this algorithm
	 */
	native private static int nativeGetPrivateKeyLength(int nativeAlgorithmId, byte[] format, byte[] encodedKey,
			int encodedKeyLength);
}
