/*
 * Java
 *
 * Copyright 2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package com.microej.vivante.utils;

import com.verisilicon.vivante.vg_lite;
import com.verisilicon.vivante.vg_lite_matrix;
import com.verisilicon.vivante.vg_lite_path;

import ej.microui.display.GraphicsContext;

/**
 * Represents a path with a colored filling.
 */
public class ColoredPath implements Path {
	private final vg_lite_path path;
	private final int color;
	private final int fillRule;

	/**
	 * Creates a colored path.
	 *
	 * @param path
	 *            the path.
	 * @param color
	 *            the ARGB color.
	 * @param fillRule
	 *            the fill rule (one of {@link vg_lite#VG_LITE_FILL_EVEN_ODD} and
	 *            {@link vg_lite#VG_LITE_FILL_NON_ZERO}).
	 */
	public ColoredPath(vg_lite_path path, int color, int fillRule) {
		this.path = path;
		this.color = color;
		this.fillRule = fillRule;
	}

	@Override
	public vg_lite_path getVGLitePath() {
		return this.path;
	}

	@Override
	public void draw(GraphicsContext g, vg_lite_matrix matrix) {
		vg_lite.vg_lite_draw_path(g, this.path, this.fillRule, matrix, vg_lite.VG_LITE_BLEND_SRC_OVER, this.color);
	}
}
