/*
 * Java
 *
 * Copyright 2019-2021 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package com.verisilicon.vivante;

/**
 * Expected to be an immutable so that it can be passed as-is to native side
 */
public class vg_lite_path {

	public static final int VG_LITE_HIGH = 0;
	public static final int VG_LITE_UPPER = 1;
	public static final int VG_LITE_MEDIUM = 2;
	public static final int VG_LITE_LOW = 3;
	public static final int VG_LITE_S8 = 0;
	public static final int VG_LITE_S16 = 1;
	public static final int VG_LITE_S32 = 2;
	public static final int VG_LITE_FP32 = 3;

	public static final int VG_LITE_CMD_CLOSE = 0;
	public static final int VG_LITE_CMD_MOVE = 2;
	public static final int VG_LITE_CMD_MOVE_REL = 3;
	public static final int VG_LITE_CMD_LINE = 4;
	public static final int VG_LITE_CMD_LINE_REL = 5;
	public static final int VG_LITE_CMD_QUAD = 6;
	public static final int VG_LITE_CMD_QUAD_REL = 7;
	public static final int VG_LITE_CMD_CUBIC = 8;
	public static final int VG_LITE_CMD_CUBIC_REL = 9;

	private float left;
	private float top;
	private float right;
	private float bottom;
	private int quality;
	private int format;
	private byte[] path;
	private int pathOffset;
	private int pathLength;

	public vg_lite_path() {
		this.left = 0;
		this.top = 0;
		this.right = 0;
		this.bottom = 0;
		this.quality = 0;
		this.format = 0;

		this.path = null;
		this.pathOffset = 0;
		this.pathLength = 0;
	}

	public vg_lite_path(float left, float top, float right, float bottom, int quality, int format, byte[] path,
			int offset, int length) {
		this.left = left;
		this.top = top;
		this.right = right;
		this.bottom = bottom;
		this.quality = quality;
		this.format = format;

		this.path = path;
		this.pathOffset = offset;
		this.pathLength = length;
	}

	public vg_lite_path(float left, float top, float right, float bottom, int quality, int format, byte[] path) {
		this(left, top, right, bottom, quality, format, path, 0, path.length);
	}

	public void update(float left, float top, float right, float bottom, int quality, int format, byte[] path,
			int offset, int length) {
		this.left = left;
		this.top = top;
		this.right = right;
		this.bottom = bottom;
		this.quality = quality;
		this.format = format;

		this.path = path;
		this.pathOffset = offset;
		this.pathLength = length;
	}

	public void update(float left, float top, float right, float bottom, int quality, int format, byte[] path) {
		this.update(left, top, right, bottom, quality, format, path, 0, path.length);
	}

	public int getQuality() {
		return this.quality;
	}

	public int getFormat() {
		return this.format;
	}

	public byte[] getPath() {
		return this.path;
	}

	public int getPathOffset() {
		return this.pathOffset;
	}

	public int getPathLength() {
		return this.pathLength;
	}

	public float getLeft() {
		return this.left;
	}

	public float getRight() {
		return this.right;
	}

	public float getTop() {
		return this.top;
	}

	public float getBottom() {
		return this.bottom;
	}
}
