/*
 * Java
 *
 * Copyright 2016-2019 MicroEJ Corp. All rights reserved.
 * MicroEJ Corp. PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package ej.ecom.wifi;

/**
 * Software-enabled Access Point (SoftAP) configuration. A SoftAP is often referred to as a virtual router. A SoftAP
 * aims at creating an access point that will expose a wireless network that clients can join.
 */
public class SoftAPConfiguration {

	private String apName;
	private String ssid;
	private String passphrase;
	private SecurityMode mode;

	/**
	 * Default constructor
	 */
	public SoftAPConfiguration() {
		super();
	}

	/**
	 * Create a new instance.
	 * 
	 * This constructor without name set the name to the ssid
	 * 
	 * @param ssid
	 *            the SSID
	 * @param passphrase
	 *            the passphrase (not be null or empty)
	 */
	public SoftAPConfiguration(String ssid, String passphrase) {
		super();
		this.ssid = ssid;
		this.passphrase = passphrase;
		this.apName = ssid;
	}

	/**
	 * Create a new instance.
	 * 
	 * @param apName
	 *            Name of soft access point
	 * @param ssid
	 *            the SSID
	 * @param passphrase
	 *            the passphrase (not be null or empty)
	 */
	public SoftAPConfiguration(String apName, String ssid, String passphrase) {
		super();
		this.ssid = ssid;
		this.passphrase = passphrase;
		this.apName = apName;
	}

	/**
	 * Get name of the SoftAP.
	 * 
	 * @return the SoftAP name
	 */
	public String getName() {
		return apName;
	}

	/**
	 * Get the SSID of the SoftAP.
	 * 
	 * @return the SoftAP SSID
	 */
	public String getSSID() {
		return ssid;
	}

	/**
	 * Get the passphrase of the SoftAP.
	 *
	 * @return the passphrase
	 */
	public String getPassphrase() {
		return passphrase;
	}

	/**
	 * Get the security mode of the SoftAP.
	 *
	 * @return the SoftAP security mode
	 */
	public SecurityMode getSecurityMode() {
		return mode;
	}

	/**
	 * Set the security mode of the SoftAP.
	 *
	 * @param the SoftAP security mode
	 */
	public void setSecurityMode(SecurityMode mode){
		this.mode = mode;
	}

	/**
	 * Set the SSID of the SoftAP.
	 *
	 * @param the SoftAP SSID
	 */
	public void setSSID(String SSID) {
		this.ssid = SSID;
	}

	/**
	 * Set the name of the SoftAP.
	 * 
	 * @param the
	 *            SoftAP name
	 */
	public void setName(String name) {
		this.apName = name;
	}

	/**
	 * Set the passphrase of the SoftAP.
	 *
	 * @param passphrase the SoftAP passphrase
	 */
	public void setPassphrase(String passphrase) {
		this.passphrase = passphrase;
	}
}
