/*
 * Java
 *
 * Copyright 2016-2020 MicroEJ Corp. All rights reserved.
 * MicroEJ Corp. PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package ej.ecom.wifi.natives;

import ej.ecom.wifi.SecurityMode;

/**
 *
 */
public class SecurityModeNativeConstants {
	/**
	 * Enterprise NO security.
	 */
	public static final int ENTERPRISE_NO_SECURITY = 0;
	/**
	 * Enterprise WPA mixed.
	 */
	public static final int ENTERPRISE_WPA_MIXED = 1;
	/**
	 * Enterprise WPA1.
	 */
	public static final int ENTERPRISE_WPA1 = 2;
	/**
	 * Enterprise WPA2.
	 */
	public static final int ENTERPRISE_WPA2 = 3;
	/**
	 * Enterprise WEP.
	 */
	public static final int ENTERPRISE_WEP = 4;
	/**
	 * MIXED (WPA1 / WPA2).
	 */
	public static final int MIXED = 5;
	/**
	 * Open (no authentification).
	 */
	public static final int OPEN = 6;
	/**
	 * WEP 128.
	 */
	public static final int WEP128 = 7;
	/**
	 * WEP 64.
	 */
	public static final int WEP64 = 8;
	/**
	 * WPA1.
	 */
	public static final int WPA1 = 9;
	/**
	 * WPA2.
	 */
	public static final int WPA2 = 10;
	/**
	 * WPA3.
	 */
	public static final int WPA3 = 11;
	/**
	 * MIXED WPA2 / WPA3.
	 */
	public static final int WPA2_WPA3_MIXED = 12;
	/**
	 * Enterprise WPA3.
	 */
	public static final int ENTERPRISE_WPA3 = 13;
	/**
	 * Enterprise MIXED WPA2 / WPA3.
	 */
	public static final int ENTERPRISE_WPA2_WPA3_MIXED = 14;
	/**
	 * Unknown (for unsupported protocols).
	 */
	public static final int UNKNOWN = 255;

	public static SecurityMode convertNativeSecurityModeToJavaMode(int mode){
		switch(mode){
		case SecurityModeNativeConstants.ENTERPRISE_NO_SECURITY:
			return SecurityMode.Enterprise_NO_security;
		case SecurityModeNativeConstants.ENTERPRISE_WPA_MIXED:
			return SecurityMode.Enterprise_WPA_mixed;
		case SecurityModeNativeConstants.ENTERPRISE_WPA1:
			return SecurityMode.Enterprise_WPA1;
		case SecurityModeNativeConstants.ENTERPRISE_WPA2:
			return SecurityMode.Enterprise_WPA2;
		case SecurityModeNativeConstants.ENTERPRISE_WEP:
			return SecurityMode.EnterpriseWEP;
		case SecurityModeNativeConstants.MIXED:
			return SecurityMode.MIXED;
		case SecurityModeNativeConstants.OPEN:
			return SecurityMode.OPEN;
		case SecurityModeNativeConstants.WEP128:
			return SecurityMode.WEP128;
		case SecurityModeNativeConstants.WEP64:
			return SecurityMode.WEP64;
		case SecurityModeNativeConstants.WPA1:
			return SecurityMode.WPA1;
		case SecurityModeNativeConstants.WPA2:
			return SecurityMode.WPA2;
		case SecurityModeNativeConstants.WPA3:
			return SecurityMode.WPA3;
		case SecurityModeNativeConstants.ENTERPRISE_WPA3:
			return SecurityMode.Enterprise_WPA3;
		case SecurityModeNativeConstants.WPA2_WPA3_MIXED:
			return SecurityMode.WPA2_WPA3_MIXED;
		case SecurityModeNativeConstants.ENTERPRISE_WPA2_WPA3_MIXED:
			return SecurityMode.Enterprise_WPA2_WPA3_MIXED;
		case SecurityModeNativeConstants.UNKNOWN:
		default:
			return SecurityMode.UNKNOWN;
		}
	}

	public static int convertJavaSecurityModeToNativeMode(SecurityMode mode) {
		switch (mode) {
		case Enterprise_NO_security:
			return ENTERPRISE_NO_SECURITY;
		case Enterprise_WPA_mixed:
			return ENTERPRISE_WPA_MIXED;
		case Enterprise_WPA1:
			return ENTERPRISE_WPA1;
		case Enterprise_WPA2:
			return ENTERPRISE_WPA2;
		case EnterpriseWEP:
			return ENTERPRISE_WEP;
		case MIXED:
			return MIXED;
		case OPEN:
			return OPEN;
		case WEP128:
			return WEP128;
		case WEP64:
			return WEP64;
		case WPA1:
			return WPA1;
		case WPA2:
			return WPA2;
		case WPA3:
			return WPA3;
		case Enterprise_WPA3:
			return ENTERPRISE_WPA3;
		case WPA2_WPA3_MIXED:
			return WPA2_WPA3_MIXED;
		case Enterprise_WPA2_WPA3_MIXED:
			return ENTERPRISE_WPA2_WPA3_MIXED;
		case UNKNOWN:
		default:
			return UNKNOWN;
		}
	}
}
