/*
 * Copyright (c) 2007,2011, Oracle and/or its affiliates. All rights reserved.
 * Copyright (C) 2015-2016, IS2T - EDC compliance and optimizations.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package java.net;

import java.io.IOException;

import com.is2t.support.net.IDatagramSocketImplFactory;

/**
 * This class defines a factory for creating DatagramSocketImpls. It defaults
 * to creating plain DatagramSocketImpls, but may create other DatagramSocketImpls
 * by setting the impl.prefix system property.
 *
 * @author Chris Hegarty
 */

public class DefaultDatagramSocketImplFactory implements IDatagramSocketImplFactory {
	static Class<?> prefixImplClass = null;

	static {
		String prefix = null;
		try {
			prefix = System.getProperty("impl.prefix");
			if (prefix != null) {
				prefixImplClass = Class.forName("java.net."+prefix+"DatagramSocketImpl");
			}
		} catch (Exception e) {
			System.err.println("Can't find class: java.net." +
					prefix +
					"DatagramSocketImpl: check impl.prefix property");
			//prefixImplClass = null;
		}
	}

	/**
	 * Creates a new <code>DatagramSocketImpl</code> instance.
	 *
	 * @param   isMulticast     true if this impl if for a MutlicastSocket
	 * @return  a new instance of a <code>DatagramSocketImpl</code>.
	 */
	@Override
	public DatagramSocketImpl createDatagramSocketImpl(boolean isMulticast /*unused on unix*/)
			throws SocketException {
		if (prefixImplClass != null) {
			try {
				return (DatagramSocketImpl)prefixImplClass.newInstance();
			} catch (Exception e) {
				throw new SocketException("can't instantiate DatagramSocketImpl");
			}
		} else {
			try {
				return new com.is2t.support.net.PlainDatagramSocketImpl(isMulticast);
			} catch (IOException e) {
				SocketException se = new SocketException();
				se.initCause(e);
				throw se;
			}
		}
	}
}
