/*
 * Java
 *
 * Copyright 2021 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package com.microej.kernel;

/**
 * Addtional features installation: cannot install additional features on
 * simulator.
 */
public class FeatureInputStream {

	/**
	 * Initializes the stack. Called once before any other function.
	 */
	public static void init() {
		// nothing to initialize
	}

	/**
	 * Checks whether a feature is available.
	 *
	 * @return <code>true</code> if the feature is available, <code>false</code>
	 *         otherwise.
	 */
	public static boolean isFeatureAvailable() {
		return false; // cannot install additional features
	}

	/**
	 * Reads a chunk of the feature into the given array.
	 *
	 * @param outOffset
	 *            write start offset in array
	 * @param length
	 *            number of bytes to read
	 * @return the number of bytes read or -1 if EOF
	 */
	public static int readIntoArray(byte[] array, int outOffset, int length) {
		throw new RuntimeException(); // never call, see isFeatureAvailable()
	}

	/**
	 * Closes the associated resources.
	 */
	public static void closeFeature() {
		throw new RuntimeException(); // never call, see isFeatureAvailable()
	}
}
